from collective.transmogrifier.transmogrifier import configuration_registry
from collective.transmogrifier.transmogrifier import Transmogrifier
from optparse import OptionParser, OptionGroup
from pkg_resources import resource_filename

import Products.GenericSetup
import logging
import mr.migrator
import sys

logging.basicConfig(level=logging.INFO)

try:
    import configparser
except ImportError:
    import ConfigParser as configparser
try:
    from Zope2.App.zcml import load_config
except:
    try:
        from Products.Five.zcml import load_config
    except:
        from zope.configuration.xmlconfig import XMLConfig as load_config
        load_config = lambda config, context: load_config(config, context)()


class Context:
    pass


class NoErrorParser(OptionParser):
    def error(self):
        pass


def runner(args={}, pipeline=None):
    # Make sure GS ZCML is loaded before we load ours

    # XXX This delays loading a bit too long. Getting:
    # ConfigurationError: ('Unknown directive',
    # u'http://namespaces.zope.org/genericsetup', u'importStep')
    # again
    #
    # load_config('autoinclude.zcml', mr.migrator)

    load_config("meta.zcml", Products.GenericSetup)
    load_config("configure.zcml", Products.GenericSetup)
    load_config('configure.zcml', mr.migrator)

    parser = OptionParser()

    parser.add_option("--pipeline", dest="pipeline",
                  help="Transmogrifier pipeline.cfg to use",
                  metavar="FILE")
    parser.add_option("--show-pipeline", dest="showpipeline",
                      action="store_true",
                      help="Show contents of the pipeline")
    parser.add_option("--zcml", dest="zcml",
                      action="store",
                      help="Load zcml")
    # Parse just the pipeline args
    ispipeline = lambda arg: [
        a for a in ['--pipeline', '--show-pipeline'] if arg.startswith(a)]
    pargs = [arg for arg in sys.argv[1:] if ispipeline(arg)]
    (options, cargs) = parser.parse_args(pargs)
    if options.pipeline is not None:
        config = options.pipeline
    elif pipeline is not None:
        config = pipeline
    else:
        # XXX How about if we look for pipeline.cfg in the cwd?
        # config = resource_filename(__name__, 'pipeline.cfg')
        config = 'pipeline.cfg'

    pipelineid = load_pipeline(config, parser)

    pargs = [arg for arg in sys.argv[1:] if not arg.startswith('--template')]
    (options, cargs) = parser.parse_args(pargs)

    cargs = {}
    for k, _, v in [a.partition('=') for a in sys.argv[1:]]:
        k = k.lstrip('--')
        if ':' in k:
            part, _, key = k.partition(':')
            if key.lower() == 'debug':
                logger = logging.getLogger(part)
                logger.setLevel(logging.DEBUG)
            else:
                section = cargs.setdefault(part, {})
                if key in section:
                    section[key] = '%s\n%s' % (section[key], v)
                else:
                    section[key] = v
        else:
            pass
            #cargs[k] = v
    for k, v in cargs.items():
        args.setdefault(k, {}).update(v)

    overrides = {}
    if type(args) == type(''):
        for arg in args:
            section, keyvalue = arg.split(':', 1)
            key, value = keyvalue.split('=', 1)
            overrides.setdefault('section', {})[key] = value
    else:
        overrides = args

    cparser = configparser.ConfigParser()
    if options.showpipeline:
        for section, values in overrides.items():
            for key, value in values.items():
                cparser.set(section, key, value)
        cparser.write(sys.stdout)
        return

    # delay this so arg processing not so slow
    load_config("meta.zcml", Products.GenericSetup)
    load_config("configure.zcml", Products.GenericSetup)

    load_config('configure.zcml', mr.migrator)

    if options.zcml:
        for zcml in options.zcml.split(','):
            if not zcml.strip():
                continue
            load_config('configure.zcml', __import__(zcml, fromlist=zcml.split('.')))

    context = Context()
    transmogrifier = Transmogrifier(context)

    transmogrifier(pipelineid, **overrides)


def load_pipeline(config, parser):
    cparser = configparser.ConfigParser()
    try:
        config_info = configuration_registry.getConfiguration(config)
        fp = open(config_info['configuration'])
        pipelineid = config
    except:
        fp = open(config)
        configuration_registry.registerConfiguration(
            u'transmogrify.config.mr.migrator',
            u"",
            u'', config)
        pipelineid = 'transmogrify.config.mr.migrator'

    try:
        # configparser
        cparser.read_file(fp)
    except:
        # ConfigParser
        cparser.read(config)

    fp.close()

    if cparser.has_option('transmogrifier', 'include'):
        load_pipeline(cparser.get('transmogrifier', 'include'), parser)

    if cparser.has_option('transmogrifier', 'pipeline'):
        pipeline = [
            p.strip() for p in cparser.get('transmogrifier', 'pipeline').split()]
    else:
        pipeline = []
    for section in pipeline:
        if section == 'transmogrifier':
            continue
        if cparser.has_option(section, '@doc'):
            doc = cparser.get(section, '@doc')
        else:
            doc = ''
        group = OptionGroup(parser, section, doc)
        for key, value in cparser.items(section):
            if key.startswith('@'):
                if key == '@doc':
                    continue
                metavar, _, help = value.partition(': ')
                if metavar.upper() == metavar:
                    action = "store"
                else:
                    action = "store_true"
                    help = value
                arg = str("--%s:%s" % (section, key[1:]))
                group.add_option(arg, action=action,
                                             help=help,
                                             metavar=metavar)
        parser.add_option_group(group)
    return pipelineid



# -*- coding: utf-8 -*-
"""Recipe mr.migrator"""

from zc.recipe.egg.egg import Scripts

import logging
import os
import re
import shutil

logging.basicConfig(level=logging.DEBUG)


class Recipe(Scripts):
    """zc.buildout recipe"""

    def __init__(self, buildout, name, options):
        self.buildout, self.name, self.options = buildout, name, options
        args = {}
        for k, v in self.options.items():
            if '-' in k:
                part, key = k.split('-', 1)
                args.setdefault(part, {})[key] = v

        self.options['scripts'] = 'migrate=%s' % name

        self.options['eggs'] = """
                mr.migrator
                """ + self.options.get('eggs', '')
        pipeline = self.options['pipeline']
        if ':' in pipeline:
            package, pipeline_name = pipeline.split(':')
            self.options['eggs'] += "  %s" % package

        pipeline = self.options.get('pipeline', None)
        if pipeline:
            self.options['arguments'] = str(args) + ',"' + pipeline + '"'
        else:
            self.options['arguments'] = str(args)

        # Process zcml
        partsdir = self.buildout['buildout']['parts-directory']
        newdir = partsdir + '/migrate'
        if not os.path.exists(newdir):
            os.mkdir(newdir)

        newdir = partsdir + '/migrate/etc'
        if not os.path.exists(newdir):
            os.mkdir(newdir)

        self.options['location'] = self.buildout['buildout']['parts-directory'] + '/migrate'
        if 'zcml' in options:
            self.build_package_includes()

        return Scripts.__init__(self, buildout, name, options)

    def install(self):
        """Installer"""
        # XXX Implement recipe functionality here
        # Return files that were created by the recipe. The buildout
        # will remove all returned files upon reinstall.
        return Scripts.install(self)

    def update(self):
        """Updater"""
        return Scripts.update(self)

    # Borrowed from p.r.zope2instance
    def build_package_includes(self):
        """Create ZCML slugs in etc/package-includes
        """
        location = self.options['location']
        sitezcml_path = os.path.join(location, 'etc', 'site.zcml')
        zcml = self.options.get('zcml')
        site_zcml = self.options.get('site-zcml')
        additional_zcml = self.options.get("zcml-additional")
        resources = self.options.get("resources")

        if site_zcml:
            open(sitezcml_path, 'w').write(site_zcml)
            return

        if zcml:
            zcml = zcml.split()

        if additional_zcml or resources or zcml:
            includes_path = os.path.join(location, 'etc', 'package-includes')

            if not os.path.exists(includes_path):
                # Zope 2.9 does not have a package-includes so we
                # create one.
                os.mkdir(includes_path)
            else:
                if zcml and '*' in zcml:
                    zcml.remove('*')
                else:
                    shutil.rmtree(includes_path)
                    os.mkdir(includes_path)

        if additional_zcml:
            path=os.path.join(includes_path, "999-additional-overrides.zcml")
            open(path, "w").write(additional_zcml.strip())

        if resources:
            resources_path = resources.strip()
            path=os.path.join(includes_path, "998-resources-configure.zcml")
            open(path, "w").write(
                resources_zcml % dict(directory=resources_path)
                )

            if not os.path.exists(resources_path):
                os.mkdir(resources_path)

        if zcml:
            n = 0
            package_match = re.compile('\w+([.]\w+)*$').match
            for package in zcml:
                n += 1
                orig = package
                if ':' in package:
                    package, filename = package.split(':')
                else:
                    filename = None

                if '-' in package:
                    package, suff = package.split('-')
                    file_suff = suff
                    if suff not in ('configure', 'meta', 'overrides'):
                        file_suff = '%s-configure' % suff
                else:
                    suff = file_suff = 'configure'

                if filename is None:
                    filename = suff + '.zcml'

                if not package_match(package):
                    raise ValueError('Invalid zcml', orig)

                path = os.path.join(
                    includes_path,
                    "%3.3d-%s-%s.zcml" % (n, package, file_suff),
                    )
                open(path, 'w').write(
                    '<include package="%s" file="%s" />\n'
                    % (package, filename)
                    )





# Add these early to avoid spurious ConfigurationErrors

try:
    eval("1 if True else 2")  # http://stackoverflow.com/questions/446052
    import Products.Five
    from Zope2.App.zcml import load_config
    load_config("configure.zcml", Products.Five)
    load_config("configure.zcml", Products.GenericSetup)
except:
    pass




import urllib
from zope.interface import Interface
from zope.schema import URI
from zope.schema import Int
from zope.schema import List
from zope.schema import Choice
from zope.schema import TextLine
from zope.schema import ASCIILine
from zope.schema import Bool
from zope.schema import Text
from zope.schema import URI
from zope.schema import Password
from zope.schema.vocabulary import SimpleVocabulary
from z3c.form import form
from z3c.form import field
from z3c.form import button
from z3c.form import group
from z3c.form import interfaces
from z3c.form.browser import multi
from plone.app.z3cform.layout import wrap_form
from collective.transmogrifier.transmogrifier import Transmogrifier
from collective.transmogrifier.transmogrifier import configuration_registry
from collective.transmogrifier.transmogrifier import _load_config
from mr.migrator.browser import MigratorMessageFactory as _
from mr.migrator.browser import logger




groupforms = {}

def formfactory(configname):
    if configname in groupforms:
        return groupforms[configname]
    groups = []
    config = _load_config(configname)
    sections = config['transmogrifier']['pipeline'].splitlines()
    print sections
    for section_id in sections:
        if not section_id:
            continue
        if section_id == 'transmogrifier':
            continue
        cparser = config[section_id]
        g = type(section_id, (group.Group,),dict(label=section_id.capitalize()))
        fields = []
        doc = cparser.get('@doc','')
        for key,value in cparser.items():
            if key in ['@doc','blueprint']:
                continue 
            if key.startswith('@'):
                key = key[1:]
                metavar,_,help = value.partition(':')
                default = unicode(cparser.get(key,''))
                help = value
            else:
                if '@'+key in cparser:
                    # let the @option line be used instead
                    continue
                else:
                    metavar = 'LINE'
                    default = unicode(value)
                    help = ''
            title = key.capitalize().replace('-',' ').replace('_',' ')
#                name = "%s:%s"%(section_id,key[1:])
            if metavar == 'HIDDEN':
                continue
            elif metavar == 'PASSWORD':
                ftype = Password()
            elif metavar == 'INT':
                ftype = Int()
                if default:
                    default = int(default)
                else:
                    default = 0
            #elif metavar == 'URL':
            #    ftype = URI()
            elif metavar == 'LIST' or '\n' in default:
                ftype = List(
                    value_type=TextLine(),)
                #ftype = Text()
                #if type(default) == type(""):
                default = default.splitlines()
                ftype.widgetFactory = multi.multiFieldWidgetFactory
            elif metavar.upper() != metavar:
                ftype = Bool()
                default = len(default)
            else:
                ftype = TextLine()
            ftype.__name__ = "%s-%s"%(section_id,key.replace('-', '_'))
            ftype.title=unicode(title)
            ftype.description=unicode(help)
            ftype.required=False
            ftype.default = default
            print (key,value,ftype,default)
            fields.append(ftype)
        if fields:
            g.fields = field.Fields(*fields)
            groups.append(g)
    groupforms[configname]=groups
    return groups

class IMigratorRun(Interface):
    """ remote source interface
    """

    config = ASCIILine  ()


class MigratorRun(group.GroupForm, form.Form):

    label = _(u"Synchronize and migrate")
    fields = field.Fields(IMigratorRun)
    ignoreContext = True

    def update(self):
        configname = self.request.get('form.widgets.config')
        self.groups = formfactory(configname)
        return super(MigratorRun, self).update()

    def updateWidgets(self):
        super(MigratorRun, self).updateWidgets()
        self.widgets['config'].mode = interfaces.HIDDEN_MODE

    @button.buttonAndHandler(u'Run')
    def handleRun(self, action):
        data, errors = self.extractData()
        if errors:
            return False

        overrides = {}
        for key, value in data.items():
            if '-' not in key:
                continue
            elif value is None:
                continue
            elif type(value) is list:
                value = '\n'.join(value)
            elif type(value) is bool:
                value = value and 'True' or 'False'
            else:
                value = str(value)
            section,key = key.split('-')
            overrides.setdefault(section, {})[key] = value

#        import ipdb; ipdb.set_trace()
        logger.info("Start importing profile: " + data['config'])
        Transmogrifier(self.context)(data['config'], **overrides)
        logger.info("Stop importing profile: " + data['config'])


class MigratorConfigurations(object):

    def __call__(self, context):
        terms = []
        for conf_id in configuration_registry.listConfigurationIds():
            print conf_id
            conf_file = _load_config(conf_id)
            for section_id in conf_file.keys():
                section = conf_file[section_id]
                if section.get('blueprint', '') == 'plone.app.transmogrifier.atschemaupdater':
                    conf = configuration_registry.getConfiguration(conf_id)
                    terms.append(SimpleVocabulary.createTerm(
                            conf_id, conf_id, conf['title']))
                    break
        return SimpleVocabulary(terms)


class IMigrator(Interface):
    """ remote source interface """

    config = Choice(
            title=_(u"Select configuration"),
            description=_(u"Registered configurations to choose from."),
            vocabulary=u"collective-migrator-configurations",
            )


class Migrator(form.Form):

    label = _(u"Synchronize and migrate")
    fields = field.Fields(IMigrator)

    ignoreContext = True

    @button.buttonAndHandler(u'Select')
    def handleSelect(self, action):
        data, errors = self.extractData()
        if errors:
            return False
        self.request.RESPONSE.redirect('%s/@@mr.migrator-run?form.widgets.%s' %
                (self.context.absolute_url(), urllib.urlencode(data)))


MigratorConfigurationsFactory = MigratorConfigurations()
MigratorRunView = wrap_form(MigratorRun)
MigratorView = wrap_form(Migrator)




# -*- coding: utf-8 -*-
"""
This module contains mr.migrator
"""
from setuptools import setup, find_packages
import os

install_requires = [
    'collective.transmogrifier',
    'setuptools',
    'z3c.autoinclude',
    'zc.recipe.egg'
    ]

try:
    # If we are using Python 2.5 or greater we can require configparser
    eval("1 if True else 2")  # http://stackoverflow.com/questions/446052
    install_requires.append('configparser')
    install_requires.append('zope.app.component')  # BBB Only needed in
                                                   # Plone >= 4?
except SyntaxError:
    # If we are using Python 2.4 or lower we cannot require configparser
    pass


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.0'

long_description = (
    read('README.rst')
    + '\n' +
    'Contributors\n'
    '************\n'
    + '\n' +
    read('CONTRIBUTORS.txt')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
   'Download\n'
    '********\n')
entry_point = ''
entry_points = {"zc.buildout": ["default = mr.migrator.recipe:Recipe"],
                'console_scripts': ['migrate = mr.migrator.runner:runner'],
                "z3c.autoinclude.plugin": ["target = plone"],
                }

tests_require = ['zope.testing', 'zc.buildout']

setup(name='mr.migrator',
      version=version,
      description="Drive-by transmogrifiction made easy!",
      long_description=long_description,
      classifiers=[
        'Framework :: Buildout',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Zope Public License',
        ],
      keywords='buildout crawler spider plone transmogrifierless',
      author='Dylan Jay',
      author_email='software@pretaweb.com',
      url='http://pypi.python.org/pypi/mr.migrator',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['mr'],
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
#      tests_require=tests_require,
#      extras_require=dict(tests=tests_require),
#      test_suite='mr.migrator.recipe.tests.test_docs.test_suite',
      entry_points=entry_points,
      )
